/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import java.sql.*;
import java.util.*;
import javax.swing.*;

/**
 * Xe[ggEt[EW[
 * @version     1.5 99/10/14
 * @author      Yuichi Omata, Sun Microsystems Inc.
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 */
public class StatementFrame {

    private final String TypeStatement = "Statement";
    private final String TypePrepared = "Prepared";
    private final String TypeCallable = "Callable";

    private Hashtable inParams = new Hashtable();
    private Hashtable outParams = new Hashtable();

    JDBCTerminal app;
    StatementFrameGUI gui;
    StatementFrameCommand command;
    InParamDialog inDialog;
    OutParamDialog outDialog;
    InspectOutDialog inspectDialog;

    Connection conn;
    String currentStmtType = TypeStatement;

    /**
     * Xe[ggEEBhEEW[
     * @param app AvP[VEW[
     */
    public StatementFrame(JDBCTerminal app){
        this.app = app;
        gui = new StatementFrameGUI(this);
        inDialog = new InParamDialog(gui, this);
        outDialog = new OutParamDialog(gui, this);
        inspectDialog = new InspectOutDialog(gui, this);
    }

    /**
     * AvP[VEW[擾B
     * @return AvP[VEW[
     */
    public JDBCTerminal getApp(){
        return app;
    }

    /**
     * Xe[ggEt[ GUI 擾B
     * @return Xe[ggEt[ GUI
     */
    public StatementFrameGUI getGUI(){
        return gui;
    }

    /**
     * Xe[ggEt[ GUI ̉ݒ肷B
     * @param b true ̂ƂAfalse ̂Ƃs
     */
    public void setVisible(boolean b){
        gui.setVisible(b);
    }

    /**
     * RlNVݒ肷B
     * @param conn RlNV
     */
    public void setConnection(Connection conn){
        this.conn = conn;
    }

    /**
     * Xe[ggEEBhE鏈sB
     */
    public void statementClose() {
        gui.setVisible(false);
        try {
            if (conn != null) {
                conn.close();
                conn = null;
            }
        } catch (SQLException ex) {
            JDBCTerminal.showMessage(
                "Cant not close connection.\n" +
                "Message: " + ex.getMessage() +"\n"+
                "State: " + ex.getSQLState() +"\n"+
                "Error code: "  + ex.getErrorCode(), gui);
        }
    }

    /**
     * SQL ̎sɎgp Statement ̎ނύXB
     * @param stmtType Statement ̎
     */
    public void statementTypeChanged(String stmtType) {
            currentStmtType = stmtType;
    }

    /**
     * SQL sB
     */
    public void exec(){
        String query = null;

        // s SQL ̎擾
        query = gui.statementArea.getSelectedText();
        if (query == null || query.equals("")){
            query = gui.statementArea.getText();
        }

        // Statement ̎ނ肵Ďs
        if (currentStmtType.equals(TypeStatement))
            executeStatement(query);
        else if (currentStmtType.equals(TypePrepared))
            executePrepared(query);
        else if (currentStmtType.equals(TypeCallable))
            executeCallable(query);            
    }

    /**
     * java.sql.Statement 𗘗p SQL sB
     * @param query sSQL 
     */
    public void executeStatement(String query) {
        try {
            boolean f;

            // Statement ̍쐬
            Statement stmt = conn.createStatement();

            // SQL ̎s
            f = stmt.execute(query);

            // ʂ̕\
            dispResult(stmt, f);
        } catch (SQLException ex) {
            JDBCTerminal.showMessage(
                "Failed SQL execution on Statement.\n" +
                "Message: " + ex.getMessage() +"\n"+
                "State: " + ex.getSQLState() +"\n"+
                "Error code: "  + ex.getErrorCode(), gui);
        }
    }

    /**
     * java.sql.PreparedStatement 𗘗p SQL sB
     * @param query sSQL 
     */
    public void executePrepared(String query) {
        try {
            // PreparedStatement ̍쐬
            PreparedStatement stmt = conn.prepareStatement(query);
            boolean f;

            // IN p[^̐ݒ
            setInParams(stmt, query);

            // SQL ̎s
            f = stmt.execute();

            // ʂ̕\
            dispResult(stmt, f);
        } catch (SQLException ex) {
           JDBCTerminal.showMessage(
               "Failed SQL execution on PreparedStatement.\n" +
               "Message: " + ex.getMessage() +"\n"+
               "State: " + ex.getSQLState() +"\n"+
               "Error code: "  + ex.getErrorCode(), gui);
       }
    }


    /**
     * IN p[^ݒ肷B
     * @param stmt p[^ݒ肷 PreparedStatement
     * @param query s SQL 
     */
    public void setInParams(PreparedStatement stmt, String query)
    throws SQLException {
        int off = 0;
        int numParam = 0;

        // IN p[^{o
        while ((off = query.indexOf('?', off)) != -1) {
            off++;
            numParam++;
        }

        // IN p[^݂ꍇ
        if (numParam > 0) {
            for (int i = 1; i < numParam + 1; i++) {
                InParam param;
                param = (InParam)inParams.get(Integer.toString(i));
                if (param != null) {
                    if ("Boolean".equals(param.getType())) {
                        stmt.setBoolean(
                            i, ((Boolean)param.getValue()).booleanValue());
                    } else if ("Byte".equals(param.getType())) {
                        stmt.setByte(i, ((Byte)param.getValue()).byteValue());
                    } else if ("Date".equals(param.getType())) {
                        stmt.setDate(
                            i, (java.sql.Date)param.getValue());
                    } else if ("Double".equals(param.getType())) {
                        stmt.setDouble(
                            i, ((Double)param.getValue()).doubleValue());
                    } else if ("Float".equals(param.getType())) {
                        stmt.setFloat(
                            i, ((Float)param.getValue()).floatValue());
                    } else if ("Int".equals(param.getType())) {
                        stmt.setInt(
                            i, ((Integer)param.getValue()).intValue());
                    } else if ("Long".equals(param.getType())) {
                        stmt.setLong(
                            i, ((Long)param.getValue()).longValue());
                    } else if ("Short".equals(param.getType())) {
                        stmt.setShort(
                            i, ((Short)param.getValue()).shortValue());
                    } else if ("String".equals(param.getType())) {
                        stmt.setString(i, (String)param.getValue());
                    } else if ("Time".equals(param.getType())) {
                        stmt.setTime(i, (java.sql.Time)param.getValue());
                    } else if ("Timestamp".equals(param.getType())) {
                        stmt.setTimestamp(
                            i, (java.sql.Timestamp)param.getValue());
                    } else if ("Null".equals(param.getType())) {
                        stmt.setNull(
                            i, InParam.getTypeFromName((String)param.getValue()));
                    }
                    continue;
                } 
            }
        }
    }

    /**
     * OUT p[^ݒ肷B
     * @param stmt p[^ݒ肷 CallableStatement
     * @param query s SQL 
     */
    public void registerOutParams(CallableStatement stmt, String query)
    throws SQLException {
        int off = 0;
        int numParam = 0;
        // OUT p[^{oB
        while ((off = query.indexOf('?', off)) != -1) {
            off++;
            numParam++;
        }
        // OUT p[^݂ꍇ
        if (numParam > 0) {
            for (int i = 1; i < numParam + 1; i++) {
                OutParam param;
                param = (OutParam)outParams.get(Integer.toString(i));
                if (param != null) {
                    stmt.registerOutParameter(i, param.getType());
                }
            }
        }
    }

    public void setOutParams(CallableStatement stmt, String query)
    throws SQLException {
        int off = 0;
        int numParam = 0;
        // OUT p[^{oB
        while ((off = query.indexOf('?', off)) != -1) {
            off++;
            numParam++;
        }
        // OUT p[^݂ꍇ
        if (numParam > 0) {
            for (int i = 1; i < numParam + 1; i++) {
                OutParam param;
                param = (OutParam)outParams.get(Integer.toString(i));
                if (param != null) {
                    int type = param.getType();
                    switch (type) {
                    case java.sql.Types.BIT:
                        param.setValue(
                            new Boolean(stmt.getBoolean(i)).toString());
                        //param.setValue(String.valueOf(stmt.getBoolean(i)));
                        //param.setValue(
                        //    ((Boolean)(stmt.getObject(i))).toString());
                        break;
                    case java.sql.Types.TINYINT:
                        param.setValue(new Byte(stmt.getByte(i)).toString());
                        //param.setValue(
                        //    ((Integer)(stmt.getObject(i))).toString());
                        break;
                    case java.sql.Types.SMALLINT:
                        param.setValue(
                             new Short(stmt.getShort(i)).toString());
                        //param.setValue(
                        //    ((Integer)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.INTEGER:
                        param.setValue(
                            new Integer(stmt.getInt(i)).toString());
                        //param.setValue(String.valueOf(stmt.getInt(i)));
                        //param.setValue(
                        //    ((Integer)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.BIGINT:
                        param.setValue(new Long(stmt.getLong(i)).toString());
                        //param.setValue(String.valueOf(stmt.getLong(i)));
                        //param.setValue(
                        //    ((Long)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.REAL:
                        param.setValue(
                            new Float(stmt.getFloat(i)).toString());
                        //param.setValue(String.valueOf(stmt.getFloat(i)));
                        //param.setValue(
                        //    ((Float)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.FLOAT:
                    case java.sql.Types.DOUBLE:
                        param.setValue(
                            new Double(stmt.getDouble(i)).toString());
                        //param.setValue(String.valueOf(stmt.getDouble(i)));
                        //param.setValue(
                        //    ((Double)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.DECIMAL:
                    case java.sql.Types.NUMERIC:
                        param.setValue(stmt.getBigDecimal(i).toString());
                        //param.setValue(
                        //    ((BigDecimal)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.CHAR:
                    case java.sql.Types.VARCHAR:
                    case java.sql.Types.LONGVARCHAR:
                        param.setValue(stmt.getString(i));
                        //param.setValue(
                        //    ((String)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.BINARY:
                    case java.sql.Types.VARBINARY:
                    case java.sql.Types.LONGVARBINARY:
                        param.setValue(new String(stmt.getBytes(i)));
                        //param.setValue(new String(stmt.getObject(i)));
                        break;
                    case java.sql.Types.DATE:
                        param.setValue(stmt.getDate(i).toString());
                        //param.setValue(
                        //    ((java.sql.Date)stmt.getObject(i)).toString());
                        break;
                    case java.sql.Types.TIME:
                        param.setValue(stmt.getTime(i).toString());
                        //param.setValue(
                        //    ((java.sql.Time)stmt.getObject(i)).toString()));
                        break;
                    case java.sql.Types.TIMESTAMP:
                        param.setValue(stmt.getTimestamp(i).toString());
                        //param.setValue(
                        //    ((java.sql.Timestamp)stmt.getObject(i)).toString());
                        break;
                    }
                    continue;
                } 
            }
        }
    }



    /**
     * java.sql.CallableStatement 𗘗p SQL sB
     * @param query sSQL 
     */
    public void executeCallable(String query) {
        try {
            // CallableStatement ̍쐬
            CallableStatement stmt = conn.prepareCall(query);
            boolean f;

            // IN p[^̐ݒ
            setInParams(stmt, query);

            // OUT p[^̓o^
            registerOutParams(stmt, query);

            // SQL ̎s
            f = stmt.execute();

            // ʂ̕\
            dispResult(stmt, f);

            // OUT p[^擾
            setOutParams(stmt, query);
        } catch (SQLException ex) {
            JDBCTerminal.showMessage(
               "Failed SQL execution on CallableStatement.\n" +
               "Message: " + ex.getMessage() +"\n"+
               "State: " + ex.getSQLState() +"\n"+
               "Error code: "  + ex.getErrorCode(), gui);
        }
    }

    /**
     * SQL sʂ\B
     * @param stmt sς Statement
     * @param frs ŏ̌ʂ ResultSet ۂ\tO
     */
    public void dispResult(Statement stmt, boolean frs) throws SQLException{
        JTextArea out = gui.resultArea;
        while (true) {
            // ʂ ResultSet 
            if (frs) {
                ResultSet rs = stmt.getResultSet();
                if (rs != null) {
                    int i;

                    // eJ̖O\邽߂ ResultSetMetaData 擾
                    ResultSetMetaData rsmd = rs.getMetaData();

                    // ResultSet Ɋ܂܂J擾
                    int numCols = rsmd.getColumnCount();

                    // J̃wb_\
                    for (i=1; i<=numCols; i++) {  
                        if (i > 1) out.append(",");
                        out.append(rsmd.getColumnLabel(i));
                    }
                    out.append("" + "\n");

                    // ResultSet Ɋ܂܂SẴf[^\
                    while (rs.next()) {
                        // eJ̕\
                        for (i=1; i<=numCols; i++) {
                            if (i > 1) out.append(",");
                            out.append(rs.getString(i));
                        }
                        out.append("" + "\n");
                        // Fetch the next result set row
                    }
                    out.append("" + "\n");
                    rs.close();
                    frs = stmt.getMoreResults();
                }
            } else {
                // ʂ update count 
                int rowCount = stmt.getUpdateCount();
                if (rowCount > 1) {
                    // ʂ update count 
                    out.append(rowCount + "rows affected.\n");
                    frs = stmt.getMoreResults();
                } else if (rowCount == 1) {
                    // ʂ update count 
                    out.append(rowCount + "row affected.\n");
                    frs = stmt.getMoreResults();
                } else if (rowCount == 0) {
                    // ǂ row ύXȂADDL 
                    out.append("0 rows affected.\n");
                    out.append("(No rows changed, "+
                               "statement was DDL command, "+
                               "or SP's return status is 0.)\n");
                    frs = stmt.getMoreResults();
                } else {
                    // NO_MORE_RESULT
                    break;
                }
            }
        } 
    } 

    /**
     * IN p[^nbVe[uɐݒ肷B
     * @param index p[^ECfbNX
     * @param value p[^̐ݒl
     */
    public void addInParam(String index, String value, String type) {
        Object obj = null;

        try {
            Integer.parseInt(index);
            if ("Boolean".equals(type))
                obj = Boolean.valueOf(value);
            else if ("Byte".equals(type))
                obj = Byte.valueOf(value);
            else if ("Date".equals(type))
                obj = java.sql.Date.valueOf(value);
            else if ("Double".equals(type))
                obj = Double.valueOf(value);
            else if ("Float".equals(type))
                obj = Float.valueOf(value);
            else if ("Int".equals(type))
                obj = Integer.valueOf(value);
            else if ("Long".equals(type))
                obj = Long.valueOf(value);
            else if ("Short".equals(type))
                obj = Short.valueOf(value);
            else if ("String".equals(type))
                obj = value;
            else if ("Time".equals(type))
                obj = java.sql.Time.valueOf(value);
            else if ("Timestamp".equals(type))
                obj = java.sql.Timestamp.valueOf(value);
            else if ("Null".equals(type))
                obj = value;

            inParams.put(index, new InParam(obj, type));
        } catch (NumberFormatException ex) {
            JDBCTerminal.showMessage("Incorrect parameter.", gui);
        }
    }

    /**
     * IN p[^NAB
     */
    public void clearInParam() {
        inParams.clear();
    }

    /**
     * IN p[^E_CAO\B
     */
    public void showInParamDialog() {
        inDialog.setVisible(true);
    }

    /**
     * OUT p[^nbVe[uɐݒ肷B
     * @param index p[^ECfbNX
     * @param value p[^̐ݒl
     */
    public void addOutParam(String index, String type) {
        try {
            Integer.parseInt(index);
            outParams.put(index, new OutParam(type));
        } catch (NumberFormatException ex) {
            JDBCTerminal.showMessage("Incorrect parameter.", gui);
        }
    }

    /**
     * OUT p[^NAB
     */
    public void clearOutParam() {
        outParams.clear();
    }

    /**
     * OUT p[^E_CAO\B
     */
    public void showOutParamDialog() {
        outDialog.setVisible(true);
    }

    /**
     * OUT p[^擾A\B
     * @param index \ OUT p[^̃CfbNX
     * @return OUT p[^̒l
     */
    public String inspectOutParam(String index) {
        return (((OutParam)(outParams.get(index))).getValue());
    }

    /**
     * OUT p[^\_CAO\B
     */
    public void showInspectDialog() {
        inspectDialog.setVisible(true);
    }

}

/**
 * IN, OUT p[^̊{NXB
 * eIN, OUT p[^QƂ^̖OƐl̑gݒ肷B
 */
class Param {
    protected static Hashtable table = new Hashtable(20);
    static {
        table.put("BIT", new Integer(java.sql.Types.BIT));
        table.put("TINYINT", new Integer(java.sql.Types.TINYINT));
        table.put("SMALLINT", new Integer(java.sql.Types.SMALLINT));
        table.put("INTEGER", new Integer(java.sql.Types.INTEGER));
        table.put("BIGINT", new Integer(java.sql.Types.BIGINT));
        table.put("FLOAT", new Integer(java.sql.Types.FLOAT));
        table.put("REAL", new Integer(java.sql.Types.REAL));
        table.put("DOUBLE", new Integer(java.sql.Types.DOUBLE));
        table.put("NUMERIC", new Integer(java.sql.Types.NUMERIC));
        table.put("DECIMAL", new Integer(java.sql.Types.DECIMAL));
        table.put("CHAR", new Integer(java.sql.Types.CHAR));
        table.put("VARCHAR", new Integer(java.sql.Types.VARCHAR));
        table.put("LONGVARCHAR", new Integer(java.sql.Types.LONGVARCHAR));
        table.put("DATE", new Integer(java.sql.Types.DATE));
        table.put("TIME", new Integer(java.sql.Types.TIME));
        table.put("TIMESTAMP", new Integer(java.sql.Types.TIMESTAMP));
        table.put("BINARY", new Integer(java.sql.Types.BINARY));
        table.put("VARBINARY", new Integer(java.sql.Types.VARBINARY));
        table.put("LONGVARBINARY", new Integer(java.sql.Types.LONGVARBINARY));
        table.put("NULL", new Integer(java.sql.Types.NULL));
    }
}

/**
 * IN p[^ENXB
 * Java ̃v~eBu^̃bpENX̎̂Ƃ̃NXB
 */
class InParam extends Param {
    Object value;
    String type;

    /**
     * RXgN^
     * @param value i[p[^ɑIuWFNg
     * @param type p[^EIuWFNg̎̂̌^
     */
    public InParam(Object value, String type) {
        this.value = value;
        this.type = type;
    }

    /**
     * p[^̒l擾B
     * @return p[^̒l
     */
    public Object getValue() {
        return value;
    }

    /**
     * p[^̌^擾B
     * @return p[^̌^
     */
    public String getType() {
        return type;
    }

    /**
     * ^炻̌^ɑCfbNXԍ擾B
     * @return Param ^ɐݒ肳Ă^CfbNXԍ
     */
    static public int getTypeFromName(String name) {
        return ((Integer)table.get(name)).intValue();
    }
}

/**
 * OUT p[^ENXB
 * p[^̌^\CfbNXԍƒlB
 */
class OutParam extends Param {

    int type;
    String value;

    /**
     * RXgN^
     * @param type i[p[^̌^CfbNXԍ
     */
    public OutParam(String type) {
        this.type = ((Integer)table.get(type)).intValue();
    }

    /**
     * i[Ăp[^̌^CfbNX擾B
     * @return p[^̌^CfbNXԍ
     */
    public int getType() {
        return type;
    }

    /**
     * p[^̒lݒ肷B
     * @param value String ^ɕϊς̐ݒ肷p[^l
     */
    public void setValue(String value) {
        this.value = value;
    }

    /**
     * p[^̒l擾B
     * @return p[^̒l
     */
    public String getValue() {
        return value;
    }
}
