/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * JDBCTerminal Xe[ggEt[ GUI
 * @version	1.2 99/10/12
 * @author 	Yuichi Omata, Sun Microsystems Inc.
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 */
public class StatementFrameGUI extends JFrame {

    StatementFrame module;
    JLabel statementAreaLabel;
    JTextArea statementArea;
    JScrollPane statementPane;
    JLabel resultAreaLabel;
    JTextArea resultArea;
    JScrollPane resultPane;
    JButton execButton;
    JPanel statementTypePanel;
    JLabel statementLabel;
    ButtonGroup statementTypeGroup;
    JCheckBox statementCheckbox;
    JCheckBox preparedCheckbox;
    JCheckBox callableCheckbox;
    JMenuBar menuBar;
    JMenu paramMenu;
    JMenuItem paramMenuSetIn;
    JMenuItem paramMenuClearIn;
    JMenuItem paramMenuSetOut;
    JMenuItem paramMenuClearOut;
    JMenuItem paramMenuInspect;

    /**
     * RXgN^
     * @param module Xe[ggEt[EW[
     */
    public StatementFrameGUI(StatementFrame module){
	this.module = module;
	addWindowListener(new StatementFrameCommand(0, module));
	setTitle("JDBCTerminal: Statement Window");

        Container pane = getContentPane();
	GridBagLayout gbl = new GridBagLayout();
	gbl.columnWidths = new int[] {14,14,14,14,14};
	gbl.rowHeights =  new int[] {14,14,14,14,14,14,14,14};
	gbl.columnWeights = new double[] {0.0,0.0,0.0,0.0,0.0};
	gbl.rowWeights = new double[] {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
	pane.setLayout(gbl);
	GridBagConstraints gbc = new GridBagConstraints();
	
        // create statement area Label
        statementAreaLabel = new JLabel("Statement:");
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbl.setConstraints(statementAreaLabel, gbc);
        pane.add(statementAreaLabel);

	// create statement TextArea
        statementArea = new JTextArea("", 5, 50);
        statementPane = new JScrollPane(
            statementArea,
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        gbc.gridy++;
	gbc.gridheight = 2;
	gbc.fill = GridBagConstraints.BOTH;
	gbl.setConstraints(statementPane, gbc);
	pane.add(statementPane);

        // create result area Label
        resultAreaLabel = new JLabel("Result:");
        gbc.gridy += 2;
        gbc.gridheight = 1;
        gbc.fill = GridBagConstraints.NONE;
	gbl.setConstraints(resultAreaLabel, gbc);
	pane.add(resultAreaLabel);

	// create result TextArea
        resultArea = new JTextArea("", 20, 50);
        resultPane = new JScrollPane(
            resultArea,
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	gbc.gridy++;
        gbc.fill = GridBagConstraints.BOTH;
	gbl.setConstraints(resultPane, gbc);
	pane.add(resultPane);

	// create exec Button
	execButton = new JButton("Exec");
	gbc.gridx = 3;
	gbc.gridy = 2;
        gbc.gridheight = 2;
	gbc.anchor = GridBagConstraints.CENTER;
	gbc.fill = GridBagConstraints.BOTH;
	gbl.setConstraints(execButton, gbc);
	execButton.addActionListener(new StatementFrameCommand(StatementFrameCommand.Exec, module));	
	pane.add(execButton);

	// create statement type Panel
	statementTypePanel = new JPanel();
	GridBagLayout stmttpGbl = new GridBagLayout();
	stmttpGbl.columnWidths = new int[] {14};
	stmttpGbl.rowHeights =  new int[] {14,14,14,14};
	stmttpGbl.columnWeights = new double[] {0.0};
	stmttpGbl.rowWeights = new double[] {0.0,0.0,0.0,0.0};
	statementTypePanel.setLayout(stmttpGbl);

	GridBagConstraints stmttpGbc = new GridBagConstraints();
	
	// create statement type Label
	statementLabel = new JLabel("Statement Type:");
	stmttpGbc.gridy = 0;
	stmttpGbc.anchor = GridBagConstraints.SOUTHWEST;
	stmttpGbl.setConstraints(statementLabel, stmttpGbc);
	statementTypePanel.add(statementLabel);

	// create checbox group
	statementTypeGroup = new ButtonGroup();

	// create checkboxes
	statementCheckbox = new JCheckBox("Statement", true);
        statementTypeGroup.add(statementCheckbox);
	stmttpGbc.gridy = 1;
	stmttpGbl.setConstraints(statementCheckbox, stmttpGbc);
	statementTypePanel.add(statementCheckbox);
        statementCheckbox.addItemListener(new StatementFrameCommand(0, module));

	preparedCheckbox = new JCheckBox("Prepared", false);
        statementTypeGroup.add(preparedCheckbox);
	stmttpGbc.gridy++;
	stmttpGbl.setConstraints(preparedCheckbox, stmttpGbc);
	statementTypePanel.add(preparedCheckbox);
        preparedCheckbox.addItemListener(new StatementFrameCommand(0, module));

	callableCheckbox = new JCheckBox("Callable", false);
        statementTypeGroup.add(callableCheckbox);
	stmttpGbc.gridy++;
	stmttpGbl.setConstraints(callableCheckbox, stmttpGbc);
	statementTypePanel.add(callableCheckbox);
        callableCheckbox.addItemListener(new StatementFrameCommand(0, module));
	
        //
	gbc.gridy += 3;
        gbc.gridheight = 1;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;
	gbl.setConstraints(statementTypePanel, gbc);
	pane.add(statementTypePanel);

        // Add menus
	menuBar = new JMenuBar();
	paramMenu = new JMenu("Parameter");
        paramMenuSetIn = new JMenuItem("set IN param");
        paramMenuSetOut = new JMenuItem("set OUT param");
        paramMenuClearIn = new JMenuItem("Clear all IN params");
        paramMenuClearOut = new JMenuItem("Clear all OUT params");
        paramMenuInspect = new JMenuItem("Inspect OUT params");
        paramMenu.add(paramMenuSetIn);
        paramMenu.add(paramMenuSetOut);
        paramMenu.add(paramMenuClearIn);
        paramMenu.add(paramMenuClearOut);
        paramMenu.add(paramMenuInspect);
        paramMenuSetIn.addActionListener(new StatementFrameCommand(StatementFrameCommand.SetIn, module));
        paramMenuSetOut.addActionListener(new StatementFrameCommand(StatementFrameCommand.SetOut, module));
        paramMenuClearIn.addActionListener(new StatementFrameCommand(StatementFrameCommand.ClearIn, module));
        paramMenuClearOut.addActionListener(new StatementFrameCommand(StatementFrameCommand.ClearOut, module));
        paramMenuInspect.addActionListener(new StatementFrameCommand(StatementFrameCommand.Inspect, module));

        menuBar.add(paramMenu);
        setJMenuBar(menuBar);

        pack();
    }

}
