/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.io.*;
import java.awt.TextComponent;
import java.awt.TextArea;
import javax.swing.JTextArea;
import javax.swing.text.JTextComponent;

/**
 * TextComponent, JTextComponent ɐڑ\ WriterB
 *  Writer ɏo͂ꂽ͂̂܂ TextComponent(JTextComponent)
 * ɕ\B\`ƂāAǉ\\łBǉ@\ OFF
 * ̏ꍇɂ́Au\ƂȂB
 * <p>
 * ܂A{ Writer ɑ΂ close() ĂяóATextComponent(JTextComponent)
 * Ƃ̐ڑӖB
 * @version	1.1 99/10/05
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 */
public class TextComponentWriter extends Writer {
    /**
     * o̓[h萔
     */
    private static final int TC_SETTEXT  = 0; // TextComponent u
    private static final int TA_APPEND   = 1; // TextArea ǉ
    private static final int TC_APPEND   = 2; // TextComponent ǉ
    private static final int JTC_SETTEXT = 3; // JTextComponent u
    private static final int JTA_APPEND  = 4; // JTextArea ǉ
    private static final int JTC_APPEND  = 5; // JTextComponent ǉ
    private static final int CLOSED      = 6; // close ς or o͕s

    /**
     * ڑ GUIB
     */
    TextComponent destTC;
    JTextComponent destJTC;

    /**
     * ǉtOB
     */
    boolean appendFlag;

    /**
     * s[hB
     */
    int outputMode;

    /**
     * TextComponent ɎRXgN^B
     * nꂽR|[lg TextArea Ȃǉ@\ ON ƂȂB
     * @param tc TextComponent IuWFNg
     */
    public TextComponentWriter(TextComponent tc) {
        this(tc, null, (tc instanceof TextArea));
    }

    /**
     * TextComponent ƒǉtOɎRXgN^B
     * @param tc TextComponent IuWFNg
     * @param ap ǉtO
     */
    public TextComponentWriter(TextComponent tc, boolean ap) {
        this(tc, null, ap);
    }

    /**
     * JTextComponent ɎRXgN^B
     * nꂽR|[lg JTextArea Ȃǉ@\ ON ƂȂB
     * @param jtc JTextComponent
     */
    public TextComponentWriter(JTextComponent jtc) {
        this(null, jtc, (jtc instanceof JTextArea));
    }

    /**
     * JTextComponent ƒǉtOɎRXgN^B
     * @param jtc JTextComponent IuWFNg
     * @param ap ǉtO
     */
    public TextComponentWriter(JTextComponent jtc, boolean ap) {
        this(null, jtc, ap);
    }

    /**
     * private RXgN^B
     * @param tc TextComponent IuWFNg
     * @param jtc JTextComponent IuWFNg
     * @param ap ǉtO
     */
    private TextComponentWriter(TextComponent tc, JTextComponent jtc,
                                boolean ap) {
        if (tc == null && jtc == null) {
            throw new IllegalArgumentException(
                "Writer ڑR|[lg null ͐ݒł܂B");
        }
        destTC = tc;
        destJTC = jtc;
        setAppendFlag(ap);
    }

    /**
     * ǉtOݒ肷B
     * @param pa ǉtO
     */
    public void setAppendFlag(boolean ap) {
        synchronized (lock) {
            appendFlag = ap;
            setupOutputMode();
        }
    }

    /**
     * ǉtOԂB
     */
    public boolean getAppendFlag() {
        return appendFlag;
    }

    /**
     * char z̕o͂B
     * @param cbuf char z
     * @param off z񂩂o͂镶Q̃ItZbg
     * @param len z񂩂o͂镶Q̒
     * @exception IOException I/O G[ꍇB
     */
    public void write(char[] cbuf, int off, int len) throws IOException {
        if (cbuf == null) {
            throw new NullPointerException();
        }
        synchronized (lock) {
            String s = new String(cbuf, off, len);
            switch (outputMode) {
            case TC_SETTEXT:
                destTC.setText(s);
                break;
            case TA_APPEND:
                ((TextArea)destTC).append(s);
                break;
            case TC_APPEND:
                destTC.setText(destTC.getText() + s);
                break;
            case JTC_SETTEXT:
                destJTC.setText(s);
                break;
            case JTA_APPEND:
                ((JTextArea)destJTC).append(s);
                break;
            case JTC_APPEND:
                destJTC.setText(destJTC.getText() + s);
                break;
            case CLOSED:
            default:
                throw new IOException(
                    "ł close Ă܂B");
            }
        }
    }

    /**
     * Writer tbVB{ Writer ł͉ȂB
     */
    public void flush() {
    }

    /**
     * Writer N[YB{ Writer ̃N[ÝAR|[lg
     * ̐؂藣ӖB
     */
    public void close() {
        synchronized (lock) {
            destTC = null;
            destJTC = null;
            setupOutputMode();
        }
        
    }

    /**
     * o̓[hݒ肷B
     */
    void setupOutputMode() {
        synchronized (lock) {
            if (destTC != null) {
                outputMode = appendFlag ?
                    ((destTC instanceof TextArea) ? TA_APPEND : TC_APPEND) :
                    TC_SETTEXT;
            } else if (destJTC != null) {
                outputMode = appendFlag ?
                    ((destJTC instanceof JTextArea) ? JTA_APPEND : JTC_APPEND) :
                    JTC_SETTEXT;
            } else {
                outputMode = CLOSED;
            }
        }
    }
}

