/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.sql.*;
import java.util.Properties;
import javax.swing.*;

/**
 * hello e[upeiXEAvP[VB
 * @author      Yuichi Omata, Tadayoshi Maekawa, Sun Microsystems K.K.
 * @version     1.3 99/10/19
 */
public class ImageStore {
    /**
     * 샂[hB
     */
    public static final int MODE_RETRIEVE = 0;
    public static final int MODE_UPDATE = 1;
    public static final int MODE_NEW = 2;
    public static final int MODE_DELETE = 3;

    /**
     * GUI ւ̃t@XB
     */
    ImageStoreUI ui;

    /**
     * f[^x[XڑB
     */
    Connection conn;

    /**
     * f[^x[XڑvpeBB
     */
    Properties dbProps;

    /**
     * C[WB
     */
    protected static final int MAX_BYTES_OF_IMAGE = 8192;

    /**
     * ݂̓샂[hB
     */
    int currentMode;

    /**
     * main \bhB
     */
    public static void main(String[] a) {
        ImageStore is = new ImageStore();
        is.start();
    }

    /**
     * RXgN^B
     */
    public ImageStore() {
        dbProps = new Properties();
        // GUI
        ui = new ImageStoreUI(this);
        ui.setVisible(true);
    }

    /**
     * NB
     */
    protected void start() {
        setupMode(MODE_RETRIEVE);
        // DB ڑ̊m
        getDBConnection();
    }

    /**
     * IB
     */
    protected void stop() {
        // DB ڑ̉
        dropDBConnection();
        if (ui != null) {
            try {
                ui.dispose();
            } catch (Exception ex) {}
            ui = null;
        }
        System.exit(0);
    }

    /**
     * 샂[h̐ݒB
     * @param md 샂[h
     */
    protected void setupMode(int md) {
        switch (md) {
        case MODE_UPDATE:
            ui.setupOperatable(MODE_UPDATE);
            break;
        case MODE_NEW:
            ui.setupOperatable(MODE_NEW);
            break;
        case MODE_DELETE:
            ui.setupOperatable(MODE_DELETE);
            break;
        case MODE_RETRIEVE:
        default:
            ui.setupOperatable(MODE_RETRIEVE);
            md = MODE_RETRIEVE;
        }
        currentMode = md;
    }

    /**
     * 샂[hɑΉ SQL sB
     * @exception InputMissException ͌肪ꍇ
     * @exception DatabaseException SQL ̎sɎsꍇ
     */
    protected void executeSQL()
    throws DatabaseException, InputMissException {
        HelloData dt = ui.getHelloData(currentMode);
        switch (currentMode) {
        case MODE_UPDATE:
            updateDB(dt);
            break;
        case MODE_NEW:
            newDB(dt);
            break;
        case MODE_DELETE:
            deleteDB(dt);
            break;
        case MODE_RETRIEVE:
        default:
            retrieveDB(dt);
        }
    }

    /**
     * Login _CAOgp DB ڑ擾B
     */
    protected void getDBConnection() {
        Connection ret;
        Login login;
        dropDBConnection();
        ui.setEnabledAll(false);
        login = new Login(ui, true, dbProps);
        ret = login.getConnection();
        if (ret != null) {
            ui.setEnabledAll(ret != null);
            ui.setupOperatable(currentMode);
        }
        conn = ret;
    }

    /**
     * w肳ꂽ DB ڑpB
     */
    protected void dropDBConnection() {
        try {
            if (conn != null) {
                conn.close();
            }
            conn = null;
        } catch (SQLException ex) {}
        ui.setEnabledAll(false);
    }

    /**
     * vpeBҏW_CAOgpāADB ڑvpeBݒ肷B
     */
    protected void setDBProperty() {
        Properties pr;
        PropertyEditor peditor;
        pr = (Properties)dbProps.clone();
        peditor = new PropertyEditor(ui, true, pr);
        pr = peditor.getProperties();
        dbProps = pr;
    }

    /**
     * hello e[ũf[^XVB
     * @param dt f[^
     * @exception DatabaseException f[^x[XANZXɎsꍇ
     */
    private void updateDB(HelloData dt) throws DatabaseException {
        try {
            PreparedStatement pstmt;
            int ud;
            pstmt = conn.prepareStatement(
                "UPDATE hello SET age = ?, birth_date = ?, "+
                "comment = ?, face = ? WHERE name = ?");
            pstmt.setInt(1, dt.age);
            pstmt.setString(2, dt.birthDate.toString()); // JDBC-ODBC bug
            pstmt.setString(3, dt.comment);
            if (dt.face != null) {
                pstmt.setBytes(4, dt.face);
            } else {
                pstmt.setNull(4, Types.LONGVARBINARY);
            }
            pstmt.setString(5, dt.name);
            ud = pstmt.executeUpdate();
            pstmt.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new DatabaseException(
                "XVɎs܂B:" + ex.getMessage());
        }
    }

    /**
     * hello e[uɃf[^o^B
     * @param dt f[^
     * @exception DatabaseException f[^x[XANZXɎsꍇ
     */
    private void newDB(HelloData dt) throws DatabaseException {
        try {
            PreparedStatement pstmt;
            int ud;
            pstmt = conn.prepareStatement(
                "INSERT hello (name, age, birth_date, comment, "+
                "face) VALUES (?, ?, ?, ?, ?)");
            pstmt.setString(1, dt.name);
            pstmt.setInt(2, dt.age);
            pstmt.setString(3, dt.birthDate.toString()); // JDBC-ODBC bug
            pstmt.setString(4, dt.comment);
            if (dt.face != null) {
                pstmt.setBytes(5, dt.face);
            } else {
                pstmt.setNull(5, Types.LONGVARBINARY);
            }
            ud = pstmt.executeUpdate();
            pstmt.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new DatabaseException(
                "o^Ɏs܂B:" + ex.getMessage());
        }
    }

    /**
     * hello e[ũf[^폜B
     * @param dt f[^
     * @exception DatabaseException f[^x[XANZXɎsꍇ
     */
    private void deleteDB(HelloData dt) throws DatabaseException {
        try {
            PreparedStatement pstmt;
            int ud;
            pstmt = conn.prepareStatement(
                 "DELETE hello WHERE name = ?");
            pstmt.setString(1, dt.name);
            ud = pstmt.executeUpdate();
            pstmt.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new DatabaseException(
                "폜Ɏs܂B:" + ex.getMessage());
        }
        dt.name = "";
        dt.age = -1;
        dt.comment = null;
        dt.face = null;
        ui.displayHelloData(dt);
    }

    /**
     * hello e[uf[^B
     * @param dt f[^
     * @exception DatabaseException f[^x[XANZXɎsꍇ
     */
    private void retrieveDB(HelloData dt) throws DatabaseException {
        try {
            PreparedStatement pstmt;
            ResultSet rs;
            pstmt = conn.prepareStatement(
                 "SELECT name, age, birth_date, comment, face "+
                 "FROM hello WHERE name = ?");
            pstmt.setString(1, dt.name);
            rs = pstmt.executeQuery();
            if (rs.next()) {
                // ef[^擾
                dt.age = rs.getInt("age");
                dt.birthDate = rs.getDate("birth_date");
                dt.comment = rs.getString("comment").trim();
                // 摜f[^擾
                InputStream fs = rs.getBinaryStream("face");
                if (rs.wasNull()) {
                    dt.face = null;
                } else {
                    int i, st, imageLen;
                    byte[] data = new byte[MAX_BYTES_OF_IMAGE];
                    imageLen = 0;
                    for (i = 0, st = fs.read(); st != -1; i++, st = fs.read()) {
                        data[i] = (byte)(st & 0x000000ff);
                    }
                    imageLen = i;
                    if (imageLen > 0) {
                        dt.face = new byte[imageLen];
                        System.arraycopy(data, 0, dt.face, 0, imageLen);
                    } else {
                        dt.face = null;
                    }
                }
            } else {
                dt.age = -1;
                dt.birthDate = null;
                dt.comment = null;
                dt.face = null;
            }
            rs.close();
            pstmt.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new DatabaseException(
                "Ɏs܂B:" + ex.getMessage());
        } catch (IOException ex) {
            ex.printStackTrace();
            throw new DatabaseException(
                "摜f[^̓ǂݍ݂Ɏs܂B:" + ex.getMessage());
        }
        ui.displayHelloData(dt);
    }
}
